@@ -1,9 +1,7 @@
-use std::cell::RefCell;
 use std::collections::HashSet;
 use std::env::Args;
+use std::os::unix::prelude::CommandExt;
 use std::process::exit;
-use std::rc::Rc;
-use std::{borrow::Cow, os::unix::prelude::CommandExt};
 use std::{env, io};
 
 use crossterm::event::{self, Event, KeyCode, KeyModifiers};
@@ -41,12 +39,12 @@ fn main() -> Result<(), io::Error> {
     execute!(stdout, EnterAlternateScreen)?;
     let backend = CrosstermBackend::new(stdout);
     let mut terminal = Terminal::new(backend)?;
-    let mut query: Cow<String> = Cow::Owned(String::new());
-    let results: Rc<RefCell<Vec<String>>> = Rc::new(RefCell::new(Vec::new()));
+    let mut query = String::new();
+    let mut results: Vec<String> = Vec::new();
     let mut mode = Mode::Insert;
     let mut selected = 0;
     let mut info_scroll = 0;
-    let mut info: Rc<RefCell<Vec<Spans>>> = Rc::new(RefCell::new(Vec::new()));
+    let mut info: Vec<Spans> = Vec::new();
     let mut redraw = true;
 
     let mut installed_cache: HashSet<usize> = HashSet::new();
@@ -54,12 +52,11 @@ fn main() -> Result<(), io::Error> {
 
     if args.query.is_some() {
         terminal.set_cursor(0, 0)?;
-        query = Cow::Owned(args.query.unwrap());
+        query = args.query.unwrap();
         let packages = search(&query, &command);
 
-        let mut res_handle = results.borrow_mut();
         for line in packages.lines() {
-            res_handle.push(line.to_owned());
+            results.push(line.to_owned());
         }
 
         mode = Mode::Select;
@@ -110,10 +107,10 @@ fn main() -> Result<(), io::Error> {
                 s.render_widget(para, chunks[0]);
 
                 let para = Paragraph::new(format_results(
-                    results.clone(),
+                    &results,
                     selected,
                     size.height as usize,
-                    (results.borrow().len() as f32 + 1f32).log10().ceil() as usize,
+                    (results.len() as f32 + 1f32).log10().ceil() as usize,
                     skipped as usize,
                     &mut installed_cache,
                     &mut cached_pages,
@@ -127,7 +124,7 @@ fn main() -> Result<(), io::Error> {
                 .alignment(Alignment::Left);
                 s.render_widget(para, chunks[1]);
 
-                if results.borrow().is_empty() {
+                if results.is_empty() {
                     let area = Rect {
                         x: size.width / 4 + 1,
                         y: size.height / 2 - 2,
@@ -156,7 +153,7 @@ fn main() -> Result<(), io::Error> {
                         width: size.width / 2,
                         height: size.height - 5,
                     };
-                    let info = Paragraph::new(if info.borrow().is_empty() {
+                    let info = Paragraph::new(if info.is_empty() {
                         {
                             let mut actions = vec![Spans::from(Span::styled(
                                 "Press ENTER to show package information",
@@ -173,7 +170,7 @@ fn main() -> Result<(), io::Error> {
                             actions
                         }
                     } else {
-                        info.borrow().iter().skip(info_scroll).cloned().collect()
+                        info.iter().skip(info_scroll).cloned().collect()
                     })
                     .block(
                         Block::default()
@@ -213,7 +210,7 @@ fn main() -> Result<(), io::Error> {
                     }
                     KeyCode::Backspace => {
                         if !query.is_empty() {
-                            query.to_mut().pop();
+                            query.pop();
                         }
                         redraw = true;
                     }
@@ -227,7 +224,7 @@ fn main() -> Result<(), io::Error> {
 
                                 return Ok(());
                             }
-                            query.to_mut().push(c);
+                            query.push(c);
                             redraw = true;
                         }
                         'w' => {
@@ -240,30 +237,29 @@ fn main() -> Result<(), io::Error> {
                                     }
                                 }
                                 let chars = chars.rev().collect::<String>();
-                                query = Cow::Owned(chars);
+                                query = chars;
                                 redraw = true;
                             } else {
-                                query.to_mut().push(c);
+                                query.push(c);
                                 redraw = true;
                             }
                         }
                         _ => {
-                            query.to_mut().push(c);
+                            query.push(c);
                             redraw = true;
                         }
                     },
                     KeyCode::Enter => {
-                        results.borrow_mut().clear();
+                        results.clear();
                         installed_cache.clear();
                         cached_pages.clear();
-                        info.borrow_mut().clear();
+                        info.clear();
                         selected = 0;
                         terminal.set_cursor(1, 4)?;
                         let packages = search(&query, &command);
 
-                        let mut res_handle = results.borrow_mut();
                         for line in packages.lines() {
-                            res_handle.push(line.to_owned());
+                            results.push(line.to_owned());
                         }
 
                         redraw = true;
@@ -281,29 +277,28 @@ fn main() -> Result<(), io::Error> {
                         } else {
                             if selected > 0 {
                                 selected -= 1;
-                                info.borrow_mut().clear();
+                                info.clear();
                             } else {
-                                selected = results.borrow().len() as u16 - 1;
-                                info.borrow_mut().clear();
+                                selected = results.len() as u16 - 1;
+                                info.clear();
                             }
                             redraw = true;
                         }
                     }
                     KeyCode::Down => {
                         if k.modifiers == KeyModifiers::CONTROL {
-                            let info = info.borrow();
                             if !info.is_empty() {
                                 info_scroll += 1;
                                 redraw = true;
                             }
                         } else {
-                            let result_count = results.borrow().len();
+                            let result_count = results.len();
                             if result_count > 1 && selected < result_count as u16 - 1 {
                                 selected += 1;
-                                info.borrow_mut().clear();
+                                info.clear();
                             } else {
                                 selected = 0;
-                                info.borrow_mut().clear();
+                                info.clear();
                             }
                             redraw = true;
                         }
@@ -311,9 +306,9 @@ fn main() -> Result<(), io::Error> {
                     KeyCode::Left => {
                         let per_page = size.height - 5;
 
-                        if selected >= per_page && results.borrow().len() > per_page as usize {
+                        if selected >= per_page && results.len() > per_page as usize {
                             selected -= per_page;
-                            info.borrow_mut().clear();
+                            info.clear();
                             redraw = true;
                         }
                     }
@@ -321,30 +316,29 @@ fn main() -> Result<(), io::Error> {
                         let size = terminal.size().unwrap();
                         let per_page = size.height - 5;
 
-                        if selected < results.borrow().len() as u16 - per_page
-                            && results.borrow().len() > per_page as usize
+                        if selected < results.len() as u16 - per_page
+                            && results.len() > per_page as usize
                         {
                             selected += per_page;
-                            info.borrow_mut().clear();
+                            info.clear();
                             redraw = true;
                         }
                     }
                     KeyCode::Char(c) => match c {
                         'j' => {
                             if k.modifiers == KeyModifiers::CONTROL {
-                                let info = info.borrow();
                                 if !info.is_empty() {
                                     info_scroll += 1;
                                     redraw = true;
                                 }
                             } else {
-                                let result_count = results.borrow().len();
+                                let result_count = results.len();
                                 if result_count > 1 && selected < result_count as u16 - 1 {
                                     selected += 1;
-                                    info.borrow_mut().clear();
+                                    info.clear();
                                 } else {
                                     selected = 0;
-                                    info.borrow_mut().clear();
+                                    info.clear();
                                 }
                                 redraw = true;
                             }
@@ -358,10 +352,10 @@ fn main() -> Result<(), io::Error> {
                             } else {
                                 if selected > 0 {
                                     selected -= 1;
-                                    info.borrow_mut().clear();
+                                    info.clear();
                                 } else {
-                                    selected = results.borrow().len() as u16 - 1;
-                                    info.borrow_mut().clear();
+                                    selected = results.len() as u16 - 1;
+                                    info.clear();
                                 }
                                 redraw = true;
                             }
@@ -370,9 +364,9 @@ fn main() -> Result<(), io::Error> {
                             let size = terminal.size().unwrap();
                             let per_page = size.height - 5;
 
-                            if selected >= per_page && results.borrow().len() > per_page as usize {
+                            if selected >= per_page && results.len() > per_page as usize {
                                 selected -= per_page;
-                                info.borrow_mut().clear();
+                                info.clear();
                                 redraw = true;
                             }
                         }
@@ -380,11 +374,11 @@ fn main() -> Result<(), io::Error> {
                             let size = terminal.size().unwrap();
                             let per_page = size.height - 5;
 
-                            if selected < results.borrow().len() as u16 - per_page
-                                && results.borrow().len() > per_page as usize
+                            if selected < results.len() as u16 - per_page
+                                && results.len() > per_page as usize
                             {
                                 selected += per_page;
-                                info.borrow_mut().clear();
+                                info.clear();
                                 redraw = true;
                             }
                         }
@@ -409,7 +403,7 @@ fn main() -> Result<(), io::Error> {
 
                                 return Ok(());
                             }
-                            query.to_mut().push(c);
+                            query.push(c);
                             redraw = true;
                         }
                         'g' => {
@@ -417,7 +411,7 @@ fn main() -> Result<(), io::Error> {
                             redraw = true;
                         }
                         'G' => {
-                            selected = results.borrow().len() as u16 - 1;
+                            selected = results.len() as u16 - 1;
                             redraw = true;
                         }
                         'R' => {
@@ -429,7 +423,7 @@ fn main() -> Result<(), io::Error> {
                                 terminal.set_cursor(0, 0)?;
                                 terminal.show_cursor()?;
                                 let mut cmd = std::process::Command::new(command);
-                                cmd.arg("-R").arg(&(results.borrow()[selected as usize]));
+                                cmd.arg("-R").arg(&(results[selected as usize]));
                                 cmd.exec();
 
                                 return Ok(());
@@ -439,13 +433,13 @@ fn main() -> Result<(), io::Error> {
                         _ => redraw = true,
                     },
                     KeyCode::Enter => {
-                        if info.borrow().is_empty() {
-                            info = Rc::new(RefCell::new(get_info(
-                                &(results.borrow()[selected as usize]),
+                        if info.is_empty() {
+                            info = get_info(
+                                &(results[selected as usize]),
                                 selected as usize,
                                 &installed_cache,
                                 &command,
-                            )));
+                            );
                             redraw = true;
                         } else {
                             disable_raw_mode()?;
@@ -455,7 +449,7 @@ fn main() -> Result<(), io::Error> {
                             terminal.set_cursor(0, 0)?;
                             terminal.show_cursor()?;
                             let mut cmd = std::process::Command::new(command);
-                            cmd.arg("-S").arg(&(results.borrow()[selected as usize]));
+                            cmd.arg("-S").arg(&(results[selected as usize]));
                             cmd.exec();
 
                             return Ok(());
@@ -477,7 +471,7 @@ fn search(query: &str, command: &str) -> String {
 }
 
 fn format_results(
-    lines: Rc<RefCell<Vec<String>>>,
+    lines: &[String],
     selected: u16,
     height: usize,
     pad_to: usize,
@@ -490,7 +484,6 @@ fn format_results(
     let installed_style = Style::default()
         .fg(Color::Green)
         .add_modifier(Modifier::BOLD);
-    // let installed_selected_style = installed_style.add_modifier(Modifier::UNDERLINED);
     let installed_selected_style = Style::default()
         .bg(Color::Red)
         .fg(Color::Yellow)
@@ -498,22 +491,13 @@ fn format_results(
     let uninstalled_style = Style::default()
         .fg(Color::LightBlue)
         .add_modifier(Modifier::BOLD);
-    // let uninstalled_selected_style = uninstalled_style.add_modifier(Modifier::UNDERLINED);
     let uninstalled_selected_style = Style::default()
         .bg(Color::Red)
         .fg(Color::Blue)
         .add_modifier(Modifier::BOLD);
 
-    let lines: Rc<Vec<String>> = Rc::new(
-        lines
-            .borrow()
-            .iter()
-            .skip(skip)
-            .take(height - 5)
-            .cloned()
-            .collect(),
-    );
-    is_installed(lines.clone(), skip, installed_cache, cached_pages, command);
+    let lines: Vec<String> = lines.iter().skip(skip).take(height - 5).cloned().collect();
+    is_installed(&lines, skip, installed_cache, cached_pages, command);
 
     let skip = skip + 1;
     lines
@@ -592,7 +576,7 @@ fn get_info(
 }
 
 fn is_installed(
-    queries: Rc<Vec<String>>,
+    queries: &[String],
     skip: usize,
     installed_cache: &mut HashSet<usize>,
     cached_pages: &mut HashSet<usize>,
@@ -604,7 +588,7 @@ fn is_installed(
 
     let mut cmd = std::process::Command::new(command);
     cmd.arg("-Qq");
-    cmd.args(queries.as_slice());
+    cmd.args(queries);
 
     let output = cmd.output().unwrap().stdout;
     let output = String::from_utf8(output).unwrap();
